@@ -3,7 +3,7 @@ use crate::rust_v0::display::{self, Style};
 use crate::rust_v0::parsers::{Context, IndexedStr};
 use crate::rust_v0::{
     Abi, BasicType, Const, DynBounds, DynTrait, DynTraitAssocBinding, GenericArg, Identifier, ImplPath, Path, Symbol,
-    Type, UndisambiguatedIdentifier,
+    Type,
 };
 use std::borrow::Cow;
 use std::rc::Rc;
@@ -11,7 +11,7 @@ use std::rc::Rc;
 fn id(disambiguator: u64, name: &str) -> Identifier {
     Identifier {
         disambiguator,
-        name: UndisambiguatedIdentifier::String(Cow::Borrowed(name)),
+        name: Cow::Borrowed(name),
     }
 }
 
@@ -31,23 +31,12 @@ fn test_parse_undisambiguated_identifier() {
     let mut parse = simplify_parser(super::parse_undisambiguated_identifier);
 
     assert_eq!(parse(""), Err(()));
-
-    assert_eq!(
-        parse("6_123foo"),
-        Ok((UndisambiguatedIdentifier::String(Cow::Borrowed("123foo")), ""))
-    );
-
-    assert_eq!(
-        parse("3bar"),
-        Ok((UndisambiguatedIdentifier::String(Cow::Borrowed("bar")), ""))
-    );
+    assert_eq!(parse("6_123foo"), Ok((Cow::Borrowed("123foo"), "")));
+    assert_eq!(parse("3bar"), Ok((Cow::Borrowed("bar"), "")));
 
     assert_eq!(
         parse("u30____7hkackfecea1cbdathfdh9hlq6y"),
-        Ok((
-            UndisambiguatedIdentifier::String(Cow::Borrowed("საჭმელად_გემრიელი_სადილი")),
-            ""
-        ))
+        Ok((Cow::Borrowed("საჭმელად_გემრიელი_სადილი"), ""))
     );
 }
 
@@ -56,16 +45,8 @@ fn test_parse_abi() {
     let mut parse = simplify_parser(super::parse_abi);
 
     assert_eq!(parse(""), Err(()));
-
     assert_eq!(parse("CDEF"), Ok((Abi::C, "DEF")));
-
-    assert_eq!(
-        parse("3abcdef"),
-        Ok((
-            Abi::Named(UndisambiguatedIdentifier::String(Cow::Borrowed("abc"))),
-            "def"
-        ))
-    );
+    assert_eq!(parse("3abcdef"), Ok((Abi::Named(Cow::Borrowed("abc")), "def")));
 }
 
 #[track_caller]
@@ -216,9 +197,9 @@ fn test_rustc_demangle_crate_with_leading_digit() {
                 path: Path::Nested {
                     namespace: b'v',
                     path: Path::CrateRoot(id(0, "123foo")).into(),
-                    name: Identifier {
+                    identifier: Identifier {
                         disambiguator: 0,
-                        name: UndisambiguatedIdentifier::String(Cow::Borrowed("bar")),
+                        name: Cow::Borrowed("bar"),
                     }
                 }
                 .into(),
@@ -239,7 +220,7 @@ fn test_rustc_demangle_utf8_idents() {
                 path: Path::Nested {
                     namespace: b'q',
                     path: Path::CrateRoot(id(0x_317d_4810_89b8_c8fe, "utf8_idents")).into(),
-                    name: id(0, "საჭმელად_გემრიელი_სადილი")
+                    identifier: id(0, "საჭმელად_გემრიელი_სადილი")
                 }
                 .into(),
                 instantiating_crate: None
@@ -263,13 +244,13 @@ fn test_rustc_demangle_closure_1() {
                         path: Path::Nested {
                             namespace: b'g',
                             path: Path::CrateRoot(id(0x_4d64_68d6_c9fd_4bb3, "cc")).into(),
-                            name: id(0, "spawn")
+                            identifier: id(0, "spawn")
                         }
                         .into(),
-                        name: id(0, "")
+                        identifier: id(0, "")
                     }
                     .into(),
-                    name: id(0, "")
+                    identifier: id(0, "")
                 }
                 .into(),
                 instantiating_crate: Some(Path::CrateRoot(id(0x_4d64_68d6_c9fd_4bb3, "cc")).into())
@@ -283,13 +264,13 @@ fn test_rustc_demangle_closure_1() {
 fn test_rustc_demangle_closure_2() {
     let crate_root = Rc::new(Path::CrateRoot(Identifier {
         disambiguator: 0x_8468_17f7_41e5_4dfd,
-        name: UndisambiguatedIdentifier::String(Cow::Borrowed("core")),
+        name: Cow::Borrowed("core"),
     }));
 
     let core_slice = Rc::new(Path::Nested {
         namespace: b'g',
         path: Rc::clone(&crate_root),
-        name: id(0, "slice"),
+        identifier: id(0, "slice"),
     });
 
     assert_eq!(
@@ -314,7 +295,7 @@ fn test_rustc_demangle_closure_2() {
                                         path: Path::Nested {
                                             namespace: b'y',
                                             path: Rc::clone(&core_slice),
-                                            name: id(0, "Iter")
+                                            identifier: id(0, "Iter")
                                         }
                                         .into(),
                                         generic_args: vec![GenericArg::Type(Type::Basic(BasicType::U8).into())]
@@ -329,18 +310,18 @@ fn test_rustc_demangle_closure_2() {
                                         path: Path::Nested {
                                             namespace: b'o',
                                             path: Rc::clone(&crate_root),
-                                            name: id(0, "iter")
+                                            identifier: id(0, "iter")
                                         }
                                         .into(),
-                                        name: id(0, "iterator")
+                                        identifier: id(0, "iterator")
                                     }
                                     .into(),
-                                    name: id(0, "Iterator")
+                                    identifier: id(0, "Iterator")
                                 }
                                 .into()
                             }
                             .into(),
-                            name: id(0, "rposition")
+                            identifier: id(0, "rposition")
                         }
                         .into(),
                         generic_args: vec![GenericArg::Type(
@@ -352,13 +333,13 @@ fn test_rustc_demangle_closure_2() {
                                         path: Path::Nested {
                                             namespace: b'p',
                                             path: Rc::clone(&core_slice),
-                                            name: id(0, "memchr")
+                                            identifier: id(0, "memchr")
                                         }
                                         .into(),
-                                        name: id(0, "memrchr")
+                                        identifier: id(0, "memrchr")
                                     }
                                     .into(),
-                                    name: id(1, "")
+                                    identifier: id(1, "")
                                 }
                                 .into()
                             )
@@ -366,7 +347,7 @@ fn test_rustc_demangle_closure_2() {
                         )]
                     }
                     .into(),
-                    name: id(0, "")
+                    identifier: id(0, "")
                 }
                 .into(),
                 instantiating_crate: Some(crate_root)
@@ -389,10 +370,10 @@ fn test_rustc_demangle_dyn_trait() {
                         path: Path::Nested {
                             namespace: b'b',
                             path: Path::CrateRoot(id(0x_f15a_878b_47eb_696b, "alloc")).into(),
-                            name: id(0, "alloc")
+                            identifier: id(0, "alloc")
                         }
                         .into(),
-                        name: id(0, "box_free")
+                        identifier: id(0, "box_free")
                     }
                     .into(),
                     generic_args: vec![GenericArg::Type(
@@ -406,17 +387,17 @@ fn test_rustc_demangle_dyn_trait() {
                                             path: Path::Nested {
                                                 namespace: b'i',
                                                 path: Path::CrateRoot(id(0x_f15a_878b_47eb_696b, "alloc")).into(),
-                                                name: id(0, "boxed")
+                                                identifier: id(0, "boxed")
                                             }
                                             .into(),
-                                            name: id(0, "FnBox")
+                                            identifier: id(0, "FnBox")
                                         }
                                         .into(),
                                         generic_args: vec![GenericArg::Type(Type::Basic(BasicType::Unit).into())]
                                     }
                                     .into(),
                                     dyn_trait_assoc_bindings: vec![DynTraitAssocBinding {
-                                        name: UndisambiguatedIdentifier::String(Cow::Borrowed("Output")),
+                                        name: Cow::Borrowed("Output"),
                                         type_: Type::Basic(BasicType::Unit).into()
                                     }]
                                 }]
@@ -445,7 +426,7 @@ fn test_rustc_demangle_const_generics_usize_123() {
                     path: Path::Nested {
                         namespace: b't',
                         path: Path::CrateRoot(id(0, "arrayvec")).into(),
-                        name: id(0, "ArrayVec")
+                        identifier: id(0, "ArrayVec")
                     }
                     .into(),
                     generic_args: vec![
@@ -478,7 +459,7 @@ fn test_rustc_demangle_const_generics_u8_11() {
                             path: Path::Nested {
                                 namespace: b't',
                                 path: Path::CrateRoot(id(0x_317d_4810_89b8_c8fe, "const_generic")).into(),
-                                name: id(0, "Unsigned")
+                                identifier: id(0, "Unsigned")
                             }
                             .into(),
                             generic_args: vec![GenericArg::Const(Const::U8(11).into())]
@@ -512,7 +493,7 @@ fn test_rustc_demangle_const_generics_i16_152() {
                             path: Path::Nested {
                                 namespace: b't',
                                 path: Path::CrateRoot(id(0x_317d_4810_89b8_c8fe, "const_generic")).into(),
-                                name: id(0, "Signed")
+                                identifier: id(0, "Signed")
                             }
                             .into(),
                             generic_args: vec![GenericArg::Const(Const::I16(152).into())]
@@ -546,7 +527,7 @@ fn test_rustc_demangle_const_generics_i8_negative_11() {
                             path: Path::Nested {
                                 namespace: b't',
                                 path: Path::CrateRoot(id(0x_317d_4810_89b8_c8fe, "const_generic")).into(),
-                                name: id(0, "Signed")
+                                identifier: id(0, "Signed")
                             }
                             .into(),
                             generic_args: vec![GenericArg::Const(Const::I8(-11).into())]
@@ -580,7 +561,7 @@ fn test_rustc_demangle_const_generics_bool_false() {
                             path: Path::Nested {
                                 namespace: b't',
                                 path: Path::CrateRoot(id(0x_317d_4810_89b8_c8fe, "const_generic")).into(),
-                                name: id(0, "Bool")
+                                identifier: id(0, "Bool")
                             }
                             .into(),
                             generic_args: vec![GenericArg::Const(Const::Bool(false).into())]
@@ -614,7 +595,7 @@ fn test_rustc_demangle_const_generics_bool_true() {
                             path: Path::Nested {
                                 namespace: b't',
                                 path: Path::CrateRoot(id(0x_317d_4810_89b8_c8fe, "const_generic")).into(),
-                                name: id(0, "Bool")
+                                identifier: id(0, "Bool")
                             }
                             .into(),
                             generic_args: vec![GenericArg::Const(Const::Bool(true).into())]
@@ -648,7 +629,7 @@ fn test_rustc_demangle_const_generics_char_v() {
                             path: Path::Nested {
                                 namespace: b't',
                                 path: Path::CrateRoot(id(0x_317d_4810_89b8_c8fe, "const_generic")).into(),
-                                name: id(0, "Char")
+                                identifier: id(0, "Char")
                             }
                             .into(),
                             generic_args: vec![GenericArg::Const(Const::Char('v').into())]
@@ -682,7 +663,7 @@ fn test_rustc_demangle_const_generics_char_lf() {
                             path: Path::Nested {
                                 namespace: b't',
                                 path: Path::CrateRoot(id(0x_317d_4810_89b8_c8fe, "const_generic")).into(),
-                                name: id(0, "Char")
+                                identifier: id(0, "Char")
                             }
                             .into(),
                             generic_args: vec![GenericArg::Const(Const::Char('\n').into())]
@@ -716,7 +697,7 @@ fn test_rustc_demangle_const_generics_char_partial_differential() {
                             path: Path::Nested {
                                 namespace: b't',
                                 path: Path::CrateRoot(id(0x_317d_4810_89b8_c8fe, "const_generic")).into(),
-                                name: id(0, "Char")
+                                identifier: id(0, "Char")
                             }
                             .into(),
                             generic_args: vec![GenericArg::Const(Const::Char('∂').into())]
@@ -754,7 +735,7 @@ fn test_rustc_demangle_const_generics_placeholder() {
                                     path: Path::Nested {
                                         namespace: b't',
                                         path: Path::CrateRoot(id(0x_317d_4810_89b8_c8fe, "const_generic")).into(),
-                                        name: id(0, "Foo")
+                                        identifier: id(0, "Foo")
                                     }
                                     .into(),
                                     generic_args: vec![GenericArg::Const(Const::Placeholder.into())]
@@ -764,10 +745,10 @@ fn test_rustc_demangle_const_generics_placeholder() {
                             .into()
                         }
                         .into(),
-                        name: id(0, "foo")
+                        identifier: id(0, "foo")
                     }
                     .into(),
-                    name: id(0, "FOO")
+                    identifier: id(0, "FOO")
                 }
                 .into(),
                 instantiating_crate: None
@@ -841,7 +822,7 @@ fn test_rustc_demangle_thinlto() {
                 path: Path::Nested {
                     namespace: b'v',
                     path: Path::CrateRoot(id(0, "backtrace")).into(),
-                    name: id(0, "foo")
+                    identifier: id(0, "foo")
                 }
                 .into(),
                 instantiating_crate: None
@@ -869,19 +850,19 @@ fn test_rustc_demangle_extra_suffix() {
                                 path: Path::Nested {
                                     namespace: 116,
                                     path: Path::CrateRoot(id(0x_693e_a8e7_2247_470f, "rand")).into(),
-                                    name: id(0, "rngs")
+                                    identifier: id(0, "rngs")
                                 }
                                 .into(),
-                                name: id(0, "adapter")
+                                identifier: id(0, "adapter")
                             }
                             .into(),
-                            name: id(0, "reseeding")
+                            identifier: id(0, "reseeding")
                         }
                         .into(),
-                        name: id(0, "fork")
+                        identifier: id(0, "fork")
                     }
                     .into(),
-                    name: id(0, "FORK_HANDLER_REGISTERED")
+                    identifier: id(0, "FORK_HANDLER_REGISTERED")
                 }
                 .into(),
                 instantiating_crate: None
